#!/usr/bin/env python

# Copyright (C) 2008 James Troup <james.troup@canonical.com>
"""whatever"""

import optparse
import sys
import types

sys.path.insert(0, "/var/lib/mailman")

from Mailman import mm_cfg
from Mailman import MailList
from Mailman import Utils
from Mailman import Errors

NAGIOS_OK=0
NAGIOS_WARN=1
NAGIOS_FAIL=2

class nagios_warning:
    def __init__(self):
        self.warning = ""
    def add(self, line):
        self.warning += line + "\n"

def check_list(listname, warning):
    config = {}
    mlist = MailList.MailList(listname, lock=0)
    for category in mm_cfg.ADMIN_CATEGORIES:
        subcats = mlist.GetConfigSubCategories(category)
        if subcats is None:
            decode_config(mlist, category, None, config)
        else:
            for subcat in [t[0] for t in subcats]:
                decode_config(mlist, category, subcat, config)

    # Rules are as follows:
    #   Canonical lists
    #     Must be archived
    #     Archives must be private
    #     Must not be advertised
    #     Subscription must require approval
    #   Ubuntu lists  host_name == lists.ubuntu.com
    #     Must be archived
    #     Archives must be publicarchive == True
    #     Must be advertised
    #     Subscription shouldn't require approval
    #       archive_private == 0

    # public lists on lists.canonical.com
    public_canonical_lists = [ "ayatana-project", "bazaar-announce", "bazaar-commits", "bazaar", "bazaar-old", "hosting-announce", "launchpad-users", "storm", "ubuntu-website" ]

    # Council lists and the like
    not_really_public = [ "community-council", "forums-council", "irc-council", "loco-council", "launchpad-buildd-admins", "technical-board", "ubuntu-membership-board-americas", "ubuntu-membership-board-asia-oceania", "ubuntu-membership-board-emea", "mailman", "ubuntu-pentest" ]
    
    # Usually lists which are too high traffic and break pipersnail
    public_but_not_archived = [ "desktop-bugs", "launchpad-buildd-admins", "ubuntu-bugs", "ubuntu-patches" ]

    # Private Canonical lists but with external (not @canonical.com) subscribers
    private_with_external = [ "intel-hardware-certification", "landscape-users", "millbank", "nick_test", "partners", "ubuntu-ps3", "uci", "utp", "dell-belmont", "canonical-codesourcery"] 

    if config["host_name"] == "lists.canonical.com" and listname not in public_canonical_lists:
        if config["advertised"] not in [ "False", "0" ]:
            print "E: private list '%s' is advertisted publicly ('%s')" % (listname, config["advertised"])
            sys.exit(NAGIOS_FAIL)
        if config["archive"] not in [ "True", "1" ]:
            print "E: private list '%s' is not archived ('%s')" % (listname, config["archive"])
            sys.exit(NAGIOS_FAIL)
        if config["archive_private"] not in [ "True", "1" ]:
            print "E: private list '%s' has public archives ('%s')" % (listname, config["archive_private"])
            sys.exit(NAGIOS_FAIL)
        if config["subscribe_policy"] not in [ "2", "3" ]:
            print "E: private list '%s' does not require subscription approval ('%s')" % (listname, config["subscribe_policy"])
            sys.exit(NAGIOS_FAIL)
        if listname not in private_with_external:
            members = mlist.getRegularMemberKeys()
            members.extend(mlist.getDigestMemberKeys())
            for member in members:
                if not member.endswith("@canonical.com"):
                    # Woo, spethial contractors
                    if member == "v.ladeuil+lp@free.fr" and listname == "canonical-bazaar":
                        continue
                    print "E: private list '%s' has non-canonical.com subscribers ('%s')" % (listname, member)
                    sys.exit(NAGIOS_FAIL)
    elif config["host_name"] == "lists.ubuntu.com" or listname in public_canonical_lists:
        if config["advertised"] not in [ "True", "1" ] and listname not in not_really_public:
            warning.add("WARN: %s@%s is not advertisted publicly ('%s')" % (listname, config["host_name"], config["advertised"]))
        if config["archive"] not in [ "True", "1" ] and listname not in public_but_not_archived:
            warning.add("WARN: %s@%s is not archived" % (listname, config["host_name"]))
        if config["archive_private"] not in [ "False", "0" ] and listname not in not_really_public:
            warning.add("WARN: %s@%s doesn't have public archives ('%s')" % (listname, config["host_name"], config["archive_private"]))
        if config["subscribe_policy"] in [ "2", "3" ] and listname not in not_really_public:
            warning.add("WARN: %s@%s requires subscription approval" % (listname, config["host_name"]))
    elif config["host_name"] in [ "lists.sa-piac.net", "lists.catherdersanon.org" ]:
        # whatever, don't care
        return
    else:
        print "E: %s has invalid host name '%s'" % (listname, config["host_name"])
        sys.exit(NAGIOS_FAIL)

def decode_config(mlist, category, subcat, config):
    info = mlist.GetConfigInfo(category, subcat)
    if not info:
        return
    for data in info[1:]:
        if not isinstance(data, types.TupleType):
            continue
        varname = data[0]
        if varname not in [ "host_name", "archive", "advertised",
                            "archive_private", "subscribe_policy" ]:
            continue
        value = getattr(mlist, varname)
        if not isinstance(value, types.StringType):
            value = repr(value)
        config[varname] = value

################################################################################

def main():
    parser=optparse.OptionParser()
    parser.add_option("-v","--verbose", action="store_true", dest="verbose")
    
    (options,args)=parser.parse_args()

    warning = nagios_warning()
                                
    names = Utils.list_names()
    names.sort()
    for n in names:
        check_list(n, warning)

    if warning.warning:
        print warning.warning,
        sys.exit(NAGIOS_WARN)

################################################################################

if __name__ == "__main__":
    main()
    
