#!/usr/bin/env python

# Check which packages are out of date
# Copyright (C) 1997-2008  James Troup <james.troup@canonical.com>

################################################################################

import os
import re
import subprocess
import sys
import tempfile

import apt_pkg

import nagios_plugin

################################################################################

re_extract_src_version = re.compile (r"(\S+)\s*\((.*)\)")

################################################################################

def read_status():
    S = {}
    Status = apt_pkg.ParseTagFile(open("/var/lib/dpkg/status"))
    while Status.Step():
        (desired, error, status) = Status.Section.Find('Status').split()
        if status != "installed":
            continue
        name = Status.Section.Find('Package')
        version = Status.Section.Find('Version')
        if S.has_key(name):
            raise nagios_plugin.CriticalError("Duplicate entry for '%s' in status file." % (name))
        S[name] = [version,desired]
    return S

################################################################################

def read_available():
    A = {}
    (temp_fd, temp_filename) = tempfile.mkstemp()
    p = subprocess.Popen(["apt-cache", "dumpavail"], stdout=temp_fd)
    ret = p.wait()
    if ret != 0:
        raise nagios_plugin.CriticalError("apt-cache dumpavail failed.")
    temp_file = open(temp_filename)
    Available = apt_pkg.ParseTagFile(temp_file)
    while Available.Step():
        name = Available.Section.Find("Package")
        version = Available.Section.Find("Version")
        source = Available.Section.Find("Source")
        filename = Available.Section.Find("Filename")
        if source and source.find("(") != -1:
            m = re_extract_src_version.match(source)
            source = m.group(1)
        if not source:
            source = name
        if A.has_key(name):
            if apt_pkg.VersionCompare(version, A[name][0]) != -1:
                continue
        A[name] = [version,source,filename]
    temp_file.close()
    os.unlink(temp_filename)
    return A

################################################################################

def main():
    apt_pkg.init()

    installed = read_status()
    available = read_available()

    out_of_date = {}
    for pkg in installed.keys():
        if not available.has_key(pkg):
            continue
        if apt_pkg.VersionCompare(installed[pkg][0], available[pkg][0]) < 0:
            (installed_version, installed_desired) = installed[pkg]
            (available_version, available_source, available_filename) = available[pkg]
            if not out_of_date.has_key(available_source):
                out_of_date[available_source] = []
            out_of_date[available_source].append([pkg,installed_version,
                                                  available_version,installed_desired])

    msg = ", ".join(out_of_date.keys())
    if len(msg) > 53:
        msg = "%s[...]" % (msg[:53])

    if msg:
        print "Packages out of-date: %s" % (msg)
        sys.exit(2)

    print "OK - Packages up-to-date."
    sys.exit(0)

################################################################################

if __name__ == "__main__":
    main()
